import {
  signTypedData,
  SignTypedDataVersion,
  TypedMessage,
} from "@metamask/eth-sig-util";

export function buildLoginSignMsg(nonce: string, tips: string) {
  const signMsg = {
    tips,
    nonce,
  };
  const signObj = {
    types: {
      EIP712Domain: [
        { name: "name", type: "string" },
        { name: "version", type: "string" },
      ],
      set: [
        { name: "tips", type: "string" },
        { name: "nonce", type: "string" },
      ],
    },
    primaryType: "set",
    domain: {
      name: "Auth",
      version: "1",
    },
    message: signMsg,
  };
  return signObj;
}

export function signLogin(nonce: string, tips: string, privateKey: string) {
  const signObj = buildLoginSignMsg(nonce, tips);
  return signTypedData({
    //@ts-ignore
    data: signObj,
    privateKey: Buffer.from(privateKey.replace("0x", ""), "hex"),
    version: SignTypedDataVersion.V4,
  });
}

const REG_KEY = /^[0-9a-fA-F]{63,64}$/;

export function isEncrypt(msg: string) {
  return !REG_KEY.test(msg);
}
